<?php
/*
 +----------------------------------------------------------------------+
 | Copyright (C) 2006 Andreas Braun                                   	|
 |									|
 | This program is free software. You can redistribute it		|
 | and/or modify it under the terms of the GNU General Public License	|
 | as published by the Free Software Foundation; either version 2       |
 | of the License, or (at your option) any later version.               |
 |                                                                      |
 | This program is distributed in the hope that it will be useful, but  |
 | WITHOUT ANY WARRANTY; without even the implied warranty of     	|
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
 | GNU General Public License for more details.     			|
 |                    							|
 +----------------------------------------------------------------------+
*/


/* ----- Functions without external parameters ----- */

//Count the number of available measuring points
function f_num(&$array, &$f_cache) {
    $f_cache['f_count'] = count($array);
    return $f_cache['f_count'];
}

//Sum
function f_sum(&$array, &$f_cache) {
    $f_cache['f_sum'] = array_sum($array);
    return $f_cache['f_sum'];
}

//Average
function f_avg(&$array, &$f_cache) {
    $f_cache['f_avg'] = array_sum($array)/count($array);
    return $f_cache['f_avg'];
}

//Maximum
function f_max(&$array, &$f_cache) {
    $f_cache['f_max'] = max($array);
    return $f_cache['f_max'];
}

//Minimum
function f_min(&$array, &$f_cache) {
    $f_cache['f_min'] = min($array);
    return $f_cache['f_min'];
}

//Gradient 
function f_grd(&$array, &$f_cache) {

    $cnt = count($array);
    
    $y_array = array_values($array);
    $x_array = array_keys($array);

    $y_i = array_sum($y_array)/$cnt;    
    $x_i = array_sum($x_array)/$cnt;
    
    $num	= 0;
    $denum	= 0;
    
    for($i=0; $i<$cnt; $i++) {
	$num	+= ($x_array[$i]-$x_i)*($y_array[$i]-$y_i);
	$denum	+= pow(($x_array[$i]-$x_i),2);
    }
    
    $f_cache['f_grd'] = $num/$denum;
    return $f_cache['f_grd'];
}




/* ----- Functions with external variables ----- */

//Xth percentitle
function f_xth( $array, &$p_cache, $value) {
    if($value > 100 || $value <= 0) return 'NA';

    sort($array);
    $x = intval(count($array)*($value/100));
    $p_cache = $array[$x-1];
    return $p_cache;
}

//Over Threshold
function f_sot(&$array, &$p_cache, $threshold) {
    
    $over_threshold = 0;
    foreach($array as $value) {
	if($value != 0) {
	    $value -= $threshold;
	    if($value > 0) $over_threshold += $value;
	}
    }
    $p_cache = $over_threshold;
    return $p_cache;
}

//Duration Over Threshold
function f_dot(&$array, &$p_cache, $threshold) {
    
    $i = 0;
    foreach($array as $value) {
	if($value != 0) {
	    $value -= $threshold;
	    if($value > 0) $i++;
	}
    }
    $p_cache = ($i/count($array))*100;
    return $p_cache;
}



/* ----- Main function for calculating ----- */

//Normal way of calulation
function calculate(& $data,& $params, & $variables, & $df_cache, & $dm_cache, & $dr_cache, & $dp_cache, & $ds_cache) {
    $results 	= array();

    $f_cache 	= $df_cache;	//Functions
    $m_cache 	= $dm_cache;    //Measurands
    $r_cache 	= $dr_cache;	//Interim results
    $p_cache	= $dp_cache;	//Functions with parameters
    $s_cache	= $ds_cache;	//Measurands with flag "spanned"

    $maxValue	= $variables['maxValue'];    
    $n_rra	= $params['rrd_ds_cnt'];
    
    //Create a cache for every Round Robin Archive  
    for($i=0; $i<$n_rra; $i++) {
	$cache[$i] = array($f_cache, $m_cache, $p_cache);
    }

    //Use reportit's error handler.
    set_error_handler('last_error');
    
    //Build the calculation command and execute it
    foreach($m_cache as $k => $m) {
	for($i=0; $i<$n_rra; $i++) {
	    
	    //Formula
	    $formula = $m;

	    //Replace our variables
	    foreach($variables as $key => $value) {
		$formula = str_replace($key, $value, $formula);
	    }

	    //Replace measurands (spanned)	    
	    foreach($s_cache as $key => $value) {
	    	if($value !== FALSE)$formula = str_replace($key, $value, $formula);	    
	    }

	    //Replace sub intermediate results first:
	    foreach($r_cache as $key => $value) {
	    	if($value !== FALSE)$formula = str_replace($key, $value, $formula);	    
	    }	    	    
	    
	    //Replace measurands with an existing result if we have one
	    foreach($cache[$i][1] as $key => $value) {
		$formula = str_replace($key, $value, $formula);	    
	    }

	    //Replace formula calls
    	    foreach($cache[$i][0] as $key => $value) {
		if($value === FALSE) {
		    $formula = str_replace($key, $key . '($data[$i], $cache[$i][0])', $formula);
		}else {
		    $formula = str_replace($key, $value, $formula);	    
		}
	    }	    

	    //Replace formula calls with parameters
	    foreach($cache[$i][2] as $key => $value) {
		if($value === FALSE) {
		    $formula = str_replace($key, $key . '( $data[$i], $cache[$i][2][$key]||', $formula);	
		    $formula = str_replace('||(', ', ', $formula);
		}else {
		    $formula = str_replace($key, $value, $formula);		    
		} 
	    }

	    eval("\$result = $formula;");
	    if($result === FALSE) $result = 'NULL';
	    
	    //If its flagged as "spanned" then update the s_cache, update the main cache 
	    //and jump to the next measurand
	    if(array_key_exists($k, $s_cache)) {
		$s_cache[$k] = $result;
		for($i=0; $i<$n_rra; $i++) {
		    unset($cache[$i][1][$k]);
		} 	
		continue 2;
	    }else {
		//Update r_cache with the result of our measurand
		$name = $k . ':' . $params['rras'][$i];
		$r_cache[$name] = $result;
	    
		//Update main cache with the result of our measurand
		$cache[$i][1][$k] = $result;
	    }
	}
    }

    //Clear up and return to main function
    $result = array();
    for($i=0; $i<$n_rra; $i++) {
	$result[] = $cache[$i][1];
    }
    //Add s_cache
    $result[] = $s_cache;
    
    //Fall back to normal error handler
    restore_error_handler();
    
    return $result;
}
?>