<?php
/*
 +----------------------------------------------------------------------+
 | Copyright (C) 2006 Andreas Braun                                   	|
 |									|
 | This program is free software. You can redistribute it		|
 | and/or modify it under the terms of the GNU General Public License	|
 | as published by the Free Software Foundation; either version 2       |
 | of the License, or (at your option) any later version.               |
 |                                                                      |
 | This program is distributed in the hope that it will be useful, but  |
 | WITHOUT ANY WARRANTY; without even the implied warranty of     	|
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
 | GNU General Public License for more details.     			|
 |                    							|
 +----------------------------------------------------------------------+
*/


//@report_data 		- Contains all input data
//@export_settings 	- Contains the globally or user specific export settings
function export_to_csv($export_data, $export_settings) {
    global $config;

    //Create some arrays
    $csv_c_sep 	= array(",", ";", "\t", " ");
    $csv_d_sep	= array(",", ".");

    //Create some variables
    $report_data	= $export_data['report_data']; 
    $report_results	= $export_data['report_results'];
    $report_measurands	= $export_data['report_measurands'];
    $report_variables	= $export_data['report_variables'];
    
    $id  		= $report_data['id'];    
    $path		= $config['base_path'] . '/plugins/reportit/exports/csv';
    
    $c_sep		= (isset($export_settings['reportit_csv_column_s']) ? $csv_c_sep[$export_settings['reportit_csv_column_s']] : ';' );
    $d_sep		= (isset($export_settings['reportit_csv_decimal_s']) ? $csv_d_sep[$export_settings['reportit_csv_decimal_s']] : ',' );
    $header		= (isset($export_settings['reportit_exp_header']) ? $export_settings['reportit_exp_header'] : '' );

    $version		= reportit_version('version');
    $info_line		= '';
    $vars		= "# Variables:\r\n";
    $legend 		= "# Legend:\r\n";    
    $tab_header_1	= '';
    $tab_header_2 	= '';    


    //Build filename and it's path
    $filename 		= str_replace("<report_id>", "$id", $export_settings['reportit_filename']) . ".csv";
    $cmp_path		= "$path/$filename";
    
    //Clear the old CSV-file 
    if(file_exists($cmp_path)) unlink($cmp_path);


    //Transform and save CSV header
    $header 		= str_replace("<cacti_version>", "Cacti: " . $config['cacti_version'], $header);
    $header		= str_replace("<reportit_version>", "Report-Plugin: " . $version, $header);

    //Build an information line
    $csv_header 	= array('Report title' 		=> "{$report_data['description']}",
				'Owner'			=> "{$report_data['owner']}",
				'Template'		=> "{$report_data['template_name']}",
				'Start'			=> "{$report_data['start_date']}",
				'End'			=> "{$report_data['end_date']}",    
				'Last Run'		=> "{$report_data['last_run']}");
    foreach($csv_header as $key => $value) $info_line = $info_line . "# $key: $value\r\n";    

    //Informations about variables
    foreach($report_variables as $var) $vars = $vars . "# {$var['name']}: {$var['value']}\r\n";

    //Build a legend for abbreviations
    foreach($report_measurands as $id) $legend = $legend . "# {$id['abbreviation']}: {$id['description']}\r\n";
  
  
    //Clean up '$report_results'
    foreach($report_results as $key => $value) {
	unset($report_results[$key]['id']);
	if(!$report_data['subhead']) unset($report_results[$key]['description']);
    }
  
    //Transform the first table header that contains the name of the datasources.
    $datasources		= explode('|', $report_data['ds_description']);
    list($rs_ids, $count_rs) 	= explode('-', $report_data['rs_def']);
    $max = 6 + $report_data['subhead'];
    for($a = 0; $a < $max; $a++) {
	$tab_header_1 = $tab_header_1 . '""' . $c_sep;
    }
    foreach($datasources as $ds) {
	for($a = 0; $a < $count_rs; $a++) {
	    $tab_header_1 = $tab_header_1 . '"'.$ds.'"'. $c_sep;
	}
    }
    
    //Transform the second one
    foreach($report_results[0] as $key => $value) {
    	if(strpos($key, '__')) {
	    list($datasource, $id) = explode('__', $key);
	    $key = $report_measurands[$id]['abbreviation'] . ' ['.$report_measurands[$id]['unit'].']';
	}
	$tab_header_2 = $tab_header_2 . '"'.$key.'"' . $c_sep;
    }  
  

    //Create a new file handle
    $filehandle 	= fopen($cmp_path, 'a');
    fwrite($filehandle, "$header\r\n\r\n");
    fwrite($filehandle, "$info_line\r\n");     
    fwrite($filehandle, "$vars\r\n");
    fwrite($filehandle, "$legend\r\n");
    fwrite($filehandle, "$tab_header_1\r\n");
    fwrite($filehandle, "$tab_header_2\r\n");    

    //Transform and save table values
    foreach($report_results as $result) {
	$row = '';
	foreach($result as $key => $value) {
	    //Check $value format
	    if (strpos($key, '__')){
		if($value === NULL) {
		    $value = 'NA';
		}else {   
		    //We should round here
		    $value = round($value, 2);
		    //Format $value with defined decimal seperator
		    $value = str_replace(".", $d_sep, $value);
		}
	    }
	    $row = $row . '"'.$value.'"' . $c_sep;
	}
        fwrite($filehandle, "$row\r\n");
    }


    //Close handle
    fclose($filehandle);
    
    return $filename;
}

?>