<?php
/*
 +----------------------------------------------------------------------+
 | Copyright (C) 2006 Andreas Braun                                   	|
 |									|
 | This program is free software. You can redistribute it		|
 | and/or modify it under the terms of the GNU General Public License	|
 | as published by the Free Software Foundation; either version 2       |
 | of the License, or (at your option) any later version.               |
 |                                                                      |
 | This program is distributed in the hope that it will be useful, but  |
 | WITHOUT ANY WARRANTY; without even the implied warranty of     	|
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
 | GNU General Public License for more details.     			|
 |                    							|
 +----------------------------------------------------------------------+
*/

$error = FALSE;

function last_error($errno, $errstr) {
    global $error;
    $error = array( 'Number'  => $errno,'Message' => $errstr);
}


function validate_calc_formula($calc_formula, $calc_intersizes, $calc_var_names) {
    global $calc_fct_names, $calc_fct_names_params;

    //Valid signs:
    $valids['intersizes']['S']	= $calc_intersizes;
    $valids['intersizes']['R']	= 'E';
    $valids['functions']['S']	= $calc_fct_names;
    $valids['functions']['R'] 	= 'E';
    $valids['fct_nwp']['S']	= $calc_fct_names_params;
    $valids['fct_nwp']['R']	= 'P';
    
    $valids['variables']['S']	= $calc_var_names;
    $valids['variables']['R']	= 'E';

    $valids['signs']['S']	= array('(', ')', '.');
    $valids['signs']['R']	= array('L', 'R', '.');
    $valids['operators']['S']	= array('+','-','*','/');
    $valids['operators']['R']	= array('+','-','*','/');
    $valids['numbers']['S']	= array('1','2','3','4','5','6','7','8','9','0');
    $valids['numbers']['R']	= 'N';


    //Invalid combinations of signs:
    $invalids			= array( '++', '+*', '+/', '--', '-*', '-/', '**', '*/', '/*', '//',
					'NE', 'EN', 
					'LR', 'L*', 'L/', '.L', 'EL', 'NL',
					'RL', '+R', '-R', '*R', '/R', 'R.', 'RE', 'RN',
					'EE', '..'
					);
    
    //Sometime it's better and easier to work with whitelists
    $whitelist			= array( 'P' => array('PL'));

    //Invalid divisions:
    $invaldiv			= array( '/0');

    //Search for invalid signs or function calls:
    $debug = '';
    $debug = $calc_formula;
    foreach($valids as $array) {
	$debug = str_replace($array['S'], '', $debug);
    }
    if(strlen($debug) != 0) return "Invalid characters: $debug"; 

    //Check the number of parenthensis;
    if(substr_count($calc_formula, '(') != substr_count($calc_formula, ')')) return "Missing parenthensis:  <span style='color:blue'>$calc_formula<span>";
    
    //Check if the formula begins or ends with an operator
    if(in_array($calc_formula[0], $valids['operators']['S'])) return "Formula begins with an operator.";
    if(in_array($calc_formula[strlen($calc_formula)-1], $valids['operators']['S'])) return "Formula ends with an operator.";
    
    //Search invalid divisions
    $debug = $calc_formula;
    foreach($invaldiv as $div) {
	$position = strpos($debug, $div);
	if($position !== FALSE) {
	    return "Division by zero:  <span style='color:blue'>$calc_formula<span>";
	}
    }


    //Search invalid combinations of operators, functions and operands:
    $debug = $calc_formula;
    foreach($valids as $array) {
	$debug = str_replace($array['S'], $array['R'], $debug);
    }

    //Blacklist
    foreach($invalids as $invalid) {
	$position = strpos($debug, $invalid);
	if($position !== FALSE) {
	    return "Syntax error b:  <span style='color:blue'>$calc_formula<span>";
	}
    }

    
    //Whitelist
    foreach($whitelist as $key => $valids) {
	$debug_w = $debug;
	$position = strpos($debug_w, $key);
	while($position !== FALSE) {

	    foreach($valids as $valid) {
		if(substr($debug_w, $position, strlen($valid)) != $valid) {
		return "Syntax error w:  <span style='color:blue'>$calc_formula<span>";
		}
	    }
	    $debug_w = substr( $debug_w, $position+1, strlen($debug_w));
	    $position = strpos($debug_w, $key);
	}
    }
        
    
    //If no error occurs the formula seems to be valid
    return 'VALID';
}



/* Safeguard against Cross-Site-Scripting */
function safeguard_xss($array = '', $exception = FALSE) {
    global $config;
    
    switch($array) {
	case "POST":
	    $array = $_POST;
	case "GET":
	    $array = $_GET;
	default:
	    $array = $_REQUEST;
    }
    
    foreach($array as $key => $value) {
	if($value != strip_tags($value, $exception) || $key != strip_tags($key, $exception)) {
	include_once($config["include_path"] . "/top_header.php");
	die_html_custom_error();
	}
    }
}



function die_html_custom_error($msg = '', $top_header = FALSE) {
    global $config;
    
    $message = '';
    $message = ($msg == '') ? 'Validation error' : $msg;
    
    if($top_header) {
	if(only_viewer()) {
	    include_once("./include/top_graph_header.php");
	}else {
	    include_once("./include/top_header.php");	
	}
    }

    echo "<table width='98%' align='center'>
	    <tr>
		<td>
		    $message
		</td>
	    </tr>
	  </table>";
    
    include_once("./include/bottom_footer.php");
    exit;
}
?>
